/**********************************************************************
* FILENAME
*   Fir_short.c
*
* DESCRIPTION
*   Fir filter with variable coefficient-field- and output-size
*	(change constants in dsp.h!!!)
* 
* Rev 2.0  Nov 07th 2001 		C. Feldbauer <feldbauer@inw.tugraz.at>
* Rev 1.0  Oct 29th 2001 		M. Rauter <rowdy@sbox.tugraz.at>
**********************************************************************/

#include "fir_short.h"


void fir_gen(short *x, short *h, short *f, int nh, int nr);
/* usually defined in fir_gen.asm (part of dsp62x.lib) or fir_gen.c */
/* but now, for convenience it is defined in THIS file!!! */


void fir_short_init( struct_fir_short *data, short *coeffs, int num_coeff, int blksz)
{
  data->coeffs = coeffs;  // coefficients
  data->num_coeff = num_coeff;  // # of FIR coefficients  
  data->blksz = blksz;    // size of output block
  data->tmpbufsz = blksz + num_coeff - 1;
  data->tmpbuf = (short *)malloc( data->tmpbufsz * sizeof( short));
  // set values zero for first block:
  memset( data->tmpbuf, 0 , data->tmpbufsz * sizeof(short));
}


void fir_short( struct_fir_short *data, short * sigbuf)
{
  int j;	
    
  for( j=0; j<data->num_coeff-1; j++)   
   {
     data->tmpbuf[j] = data->tmpbuf[data->blksz+j];    // copy old data
   }  
  
  for( j=data->num_coeff-1; j<data->tmpbufsz; j++)  
   {
     data->tmpbuf[j] = sigbuf[j-(data->num_coeff-1)];  // copy new data
   } 
  
  // calculate convolution:
  fir_gen( data->tmpbuf, data->coeffs, sigbuf, data->num_coeff, data->blksz);
}






/*
 * This is the C equivalent of fir_gen.asm.
 */

void fir_gen( short x[], short h[], short r[], int nh, int nr)
{
  int i, j, sum;

  for( j = 0; j < nr; j++)
  {
    sum = 0;
    for( i = 0; i < nh; i++)
      sum += x[i + j] * h[i];
    r[j] = sum >> 15;
  }
}
